from django.http import HttpResponse
from django.utils import simplejson
from decimal import Decimal
import datetime


class BetterJSONEncoder(simplejson.JSONEncoder):
    """JSON encoder which understands decimals, dates, and __json__"""

    def default(self, obj):
        '''Convert object to JSON encodable type.'''
        if hasattr(obj, '__json__'):
            return obj.__json__()
        if isinstance(obj, Decimal):
            return "%d" % obj
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        return simplejson.JSONEncoder.default(self, obj)


class APIError(Exception):
    def __init__(self, code, errors, *args, **kwargs):
        self.code = code
        self.errors = errors
        super(APIError, self).__init__(*args, **kwargs)


def return_error(code, errors):
    resp = HttpResponse(simplejson.dumps(errors),
            mimetype="application/json")
    resp.status_code = code
    return resp


def jsonapi(**decor_kwargs):
    def fnc(fn):
        def to_json(request, *args, **kwargs):

            # check allowed methods if set
            allowed_methods = decor_kwargs.get("allowed_methods", ["POST", "GET", "PUT", "DELETE"])
            if request.method.lower() not in [ am.lower() for am in allowed_methods ]:
                return return_error(403, "Method %s not allowed" % request.method)

            # check for APIErrors
            try:
                context_data = fn(request, *args, **kwargs)
            except APIError, apie:
                return return_error(apie.code, apie.errors)

            # return JSON if a dict is returned, else return an httpresponse
            if isinstance(context_data, HttpResponse):
                return context_data
            resp = HttpResponse(simplejson.dumps(context_data, cls=BetterJSONEncoder),
                    mimetype='application/json')
            resp.status_code = { "POST" : 201, "GET" : 200, "DELETE" : 204, "PUT" : 201 }[request.method]

            return resp

        return to_json
    return fnc



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'chunk'
        db.create_table('chunky_chunk', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('content', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('chunky', ['chunk'])


    def backwards(self, orm):
        # Deleting model 'chunk'
        db.delete_table('chunky_chunk')


    models = {
        'chunky.chunk': {
            'Meta': {'object_name': 'chunk'},
            'content': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['chunky']


from django import template
from chunky.models import Chunk

register = template.Library()



@register.inclusion_tag('chunk_template.html', takes_context = True)
def chunk(context, slug):

    if not slug:
        raise Exception("""
        You have to have a slug in your 'chunk' tag.
        Note: {% chunk foo_bar %} is resolving 'foo_bar' as a variable.
        Normal usage is {% chunk "foo_bar" %}
        """)

    chunk, created = Chunk.objects.get_or_create(slug=slug)

    return {
        "content":chunk.content,
        "slug":chunk.slug,
    }




from setuptools import setup

import os

def get_source_files():
    source_files = []
    for dirname, _, files in os.walk('chunky'):
        for filename in files:
            if not filename.endswith('.pyc'):
                source_files.append( os.path.join('/'.join(dirname.split('/')[1:]), filename))
    return source_files

setup(
    name='django-chunky',
    version='0.1.5',
    description='Editable content chunks for Django',
    author='Chris Pitzer, Jeff Schwaber, LoFi Art',
    author_email='freyley@gmail.com',
    url='http://github.com/freyley/django-chunky',
    packages=['chunky'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    zip_safe=False,
    #test_suite='tests.runtests.runtests',
        

)



from django.db import models


class Chunk(models.Model):
    slug = models.CharField(unique=True, max_length=255)
    content = models.TextField(default="THIS CHUNK HAS NO CONTENT")

    def __unicode__(self):
        return self.slug



from models import Chunk
from . import decorators


def user_test(user):
    return user.is_superuser

@decorators.jsonapi(allowed_methods=["POST"])
def edit_api(request):
    if not request.user.is_superuser:
        raise decorators.APIError(403, "Only logged in superusers can do this. Please log in.")
    try:
        chunk, created = Chunk.objects.get_or_create(slug=request.POST["slug"])
        chunk.content = request.POST["content"]
        chunk.save()
        return { "success": True }
    except Exception, e:
        raise decorators.APIError(500, str(e))
